#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2024/3/19 21:15
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""
PythonAPI：https://help.marine.copernicus.eu/en/articles/8286883-copernicus-marine-toolbox-api-get-original-files
PythonOpendap: https://help.marine.copernicus.eu/en/articles/8287609-copernicus-marine-toolbox-api-open-a-dataset-or-read-a-dataframe-remotely
PythonCSV: https://help.marine.copernicus.eu/en/articles/7970637-how-to-download-data-for-multiple-points-from-a-csv
PythonRecurringperiod: https://help.marine.copernicus.eu/en/articles/7973161-how-to-get-data-for-a-recurring-period
"""
